@@ -1,8 +1,8 @@
 /* cmdline.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -905,6 +905,9 @@ restart(int sig)
 	console("restart (%s) %s\n", signame(sig), 
 		pc->flags & IN_GDB ? "(in gdb)" : "(in crash)");
 
+	if (sig == SIGUSR2)
+		clean_exit(1);
+
         if (pc->flags & IN_RESTART) {
                 fprintf(stderr, 
 		   "\nembedded signal received (%s): recursive restart call\n",
@@ -1162,9 +1165,11 @@ restore_ifile_sanity(void)
         }
 
         if (pc->flags & TTY) {
-                if ((fd = open("/dev/tty", O_RDONLY)) < 0) 
-			error(FATAL, "/dev/tty: %s\n", strerror(errno));
-                
+                if ((fd = open("/dev/tty", O_RDONLY)) < 0) {
+                        console("/dev/tty: %s\n", strerror(errno));
+                        clean_exit(1);
+                }
+ 
                 if (tcsetattr(fd, TCSANOW, &pc->termios_orig) == -1) 
 			error(FATAL, "tcsetattr /dev/tty: %s\n",
                                 strerror(errno));
