@@ -1,6 +1,7 @@
 /* Definitions for frame address handler, for GDB, the GNU debugger.
 
-   Copyright (C) 2003, 2004, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -72,6 +73,7 @@ frame_base_init (struct obstack *obstack)
 {
   struct frame_base_table *table
     = OBSTACK_ZALLOC (obstack, struct frame_base_table);
+
   table->tail = &table->head;
   table->default_base = &default_frame_base;
   return table;
@@ -82,7 +84,9 @@ frame_base_append_sniffer (struct gdbarch *gdbarch,
 			   frame_base_sniffer_ftype *sniffer)
 {
   struct frame_base_table *table = gdbarch_data (gdbarch, frame_base_data);
-  (*table->tail) = GDBARCH_OBSTACK_ZALLOC (gdbarch, struct frame_base_table_entry);
+
+  (*table->tail)
+    = GDBARCH_OBSTACK_ZALLOC (gdbarch, struct frame_base_table_entry);
   (*table->tail)->sniffer = sniffer;
   table->tail = &(*table->tail)->next;
 }
@@ -92,6 +96,7 @@ frame_base_set_default (struct gdbarch *gdbarch,
 			const struct frame_base *default_base)
 {
   struct frame_base_table *table = gdbarch_data (gdbarch, frame_base_data);
+
   table->default_base = default_base;
 }
 
@@ -105,6 +110,7 @@ frame_base_find_by_frame (struct frame_info *this_frame)
   for (entry = table->head; entry != NULL; entry = entry->next)
     {
       const struct frame_base *desc = NULL;
+
       desc = entry->sniffer (this_frame);
       if (desc != NULL)
 	return desc;
